from Hero import *
from GameMap import *
from Startbox import *

"""
CONSTANTS
"""
# Game constants
WHITE = (255, 255, 255)
GAME_TITLE = "Jeff's Lair"
SCREEN_SIZE = (800, 600)
GAME_SPEED = 60  # game speed in fps

# timer
COUNTER_START = 100  # number of seconds to complete maze
ONE_SECOND = 1000  # number of ticks in a second for timer
TIMER_FONT_SIZE = 24
TIMER_LOCATION = (8, 8)
TIMER_TEXT = "Time remaining: "

# Building the maze
MAZE_WIN_IMG = "./resources/images/1_win.png"
MAZE_LOSE_IMG = "./resources/images/1_lose.png"
NICKNAME_LOCATION = (280, 350)
NICKNAME_FONT_SIZE = 40
NICKNAME_TEXT = "Well done "
NICKNAME_LEADING = 20
MAZE_MAP_IMG = "./resources/images/1_maze.png"
MAZE_MAP = "./resources/maps/1_maze.txt"
CHARACTER_IMG = "./resources/images/1_char.png"

WIN_BLOCK_FROM_END = 26  # defines the win square
HERO_TILE_SIZE = 32  # size in px of main player height and width
GAME_TILE_SIZE = 32   # size in px of main grid height and width
PLAYER_BORN_X = 1 * HERO_TILE_SIZE  # x position where hero starts in px
PLAYER_BORN_Y = 1 * HERO_TILE_SIZE  # y position where hero starts in px

RAT_IMG = "./resources/images/rat.png"
RAT_BORN_X = 23 * HERO_TILE_SIZE  # x position where rat starts in px
RAT_BORN_Y = 17 * HERO_TILE_SIZE  # y position where rat starts in px
SLOW_RAT = 10  # make rat update every x ms


class Game:
    def __init__(self, title, screen_width, screen_height, nickname, fps=GAME_SPEED):
        """
        25 columns width and 19 columns height
        :param title: title of the game
        :param screen_width: screen width
        :param screen_height: screen height
        :param fps: refresh times per second
        """
        # data for pygame
        self.title = title
        self.screen_width = screen_width
        self.screen_height = screen_height
        self.fps = fps
        self.screen = None
        self.nickname = nickname

        # data for game
        self.hero_born_x = PLAYER_BORN_X
        self.hero_born_y = PLAYER_BORN_Y
        self.hero_width = HERO_TILE_SIZE
        self.hero_height = HERO_TILE_SIZE

        # run functions below
        self.__init_pygame()
        self.__init_game()
        self.update()

    def __init_pygame(self):
        pygame.init()
        pygame.display.set_caption(self.title)
        self.screen = pygame.display.set_mode((self.screen_width, self.screen_height))
        self.clock = pygame.time.Clock()

    def __init_game(self):
        global BLOCK_GROUP
        # initialise map
        self.maze_win_img = pygame.image.load(MAZE_WIN_IMG).convert_alpha()
        self.maze_lose_img = pygame.image.load(MAZE_LOSE_IMG).convert_alpha()
        self.game_map_img = pygame.image.load(MAZE_MAP_IMG).convert_alpha()
        self.game_map = GameMap(self.game_map_img, 0, 0)
        self.game_block_group = self.game_map.make_block_group(MAZE_MAP, GAME_TILE_SIZE)
        # initialise hero
        self.hero_fulimg = pygame.image.load(CHARACTER_IMG).convert_alpha()
        self.hero_rect = Rect(self.hero_born_x, self.hero_born_y, self.hero_width, self.hero_height)
        self.hero = Hero(self.hero_fulimg, self.hero_rect, GAME_TILE_SIZE)
        # others
        self.font = pygame.font.Font(None, TIMER_FONT_SIZE)

        # initialise rat
        self.rat_fulimg = pygame.image.load(RAT_IMG).convert_alpha()

        # start rat at particular space
        self.rat_rect = Rect(RAT_BORN_X, RAT_BORN_Y, GAME_TILE_SIZE, GAME_TILE_SIZE)

        # pretty sure that the rat_rect includes the x,y for start, not the last 2 params of Hero
        self.rat = Hero(self.rat_fulimg, self.rat_rect, GAME_TILE_SIZE)


    def collide(self, hero_rect1, hero_rect2, GAME_TILE_SIZE):
        is_collision = False
        # hero/monster conflict LEFT
        if hero_rect1.top >= hero_rect2.top - GAME_TILE_SIZE and hero_rect1.bottom <= hero_rect2.bottom + GAME_TILE_SIZE:
            if hero_rect1.right >= hero_rect2.right >= hero_rect1.left:
                is_collision = True
        # right
        if hero_rect1.top >= hero_rect2.top - GAME_TILE_SIZE and hero_rect1.bottom <= hero_rect2.bottom + GAME_TILE_SIZE:
            if hero_rect1.left <= hero_rect2.left <= hero_rect1.right:
                is_collision = True

        # up
        if hero_rect1.left >= hero_rect2.left - GAME_TILE_SIZE and hero_rect1.right <= hero_rect2.right + GAME_TILE_SIZE:
            if hero_rect1.bottom >= hero_rect2.bottom >= hero_rect1.top:
                is_collision = True

        if hero_rect1.left >= hero_rect2.left - GAME_TILE_SIZE and hero_rect1.right <= hero_rect2.right + GAME_TILE_SIZE:
            if hero_rect1.top <= hero_rect2.top <= hero_rect1.bottom:
                is_collision = True
        return is_collision


    def update(self):

        # initialise counter
        counter = COUNTER_START

        # counts time elapsed in ticks
        last_time = 0

        # counter to delay rat moves
        last_rat_move = 0

        # is the game over yet ?
        game_over = False
        lose_game = False

        # main game loop
        while 1:
            if game_over:
                # show the win background
                self.screen.blit(self.maze_win_img, (0, 0))
                # show the winner name and time remaining
                self.screen.blit(self.font.render(NICKNAME_TEXT + self.nickname, True, WHITE), NICKNAME_LOCATION)
                self.screen.blit(self.font.render(text_clock, True, WHITE), (NICKNAME_LOCATION[0], NICKNAME_LOCATION[1]+NICKNAME_LEADING))
                # print(self.nickname)
                pygame.display.update()
                for m in pygame.event.get():
                    if m.type == QUIT:
                        exit()
                    if m.type == KEYDOWN:
                        exit()
                self.clock.tick(self.fps)
                continue
            if lose_game:
                self.screen.blit(self.maze_lose_img, (0, 0))  # win bg
                pygame.display.update()
                for m in pygame.event.get():
                    if m.type == QUIT:
                        exit()
                    if m.type == KEYDOWN:
                        exit()
                self.clock.tick(self.fps)
                continue
            # game is playing
            for event in pygame.event.get():
                if event.type == QUIT:
                    exit()

            # draw the map
            self.game_map.draw_map(self.screen)

            # Timer. Check for 1000ms passed since last_time then reduce counter by one
            ticks = pygame.time.get_ticks()
            if ticks > last_time + ONE_SECOND:
                last_time = ticks
                counter -= 1

            # blit Timer to the screen
            text_clock = TIMER_TEXT + str(counter)
            self.screen.blit(self.font.render(text_clock, True, WHITE), TIMER_LOCATION)

            # Only move rat slower every x ms
            move_rat = False
            if ticks > last_rat_move + SLOW_RAT:
                move_rat = True
                last_rat_move = ticks
            self.rat.monster_move(self.screen, self.hero_rect, self.game_block_group, move_rat, GAME_TILE_SIZE)

            # logic update
            press_keys = pygame.key.get_pressed()
            game_over = self.hero.hero_moving(self.screen, press_keys, self.game_block_group, SCREEN_SIZE[0], SCREEN_SIZE[1], WIN_BLOCK_FROM_END, GAME_TILE_SIZE)

            # collision detection
            lose_game = self.collide(self.rat_rect, self.hero_rect, GAME_TILE_SIZE)

            pygame.display.update()
            self.clock.tick(self.fps)






from pygame.locals import *

class Hero:
    def __init__(self, hero_fulimg, hero_rect, GAME_TILE_SIZE):
        # Hero image
        self.hero_fulimg = hero_fulimg

        # attach image to surface
        self.hero_img = [self.hero_fulimg.subsurface(Rect((0, 0), (GAME_TILE_SIZE, GAME_TILE_SIZE)))]
        self.hero_rect = hero_rect
        self.hero_speed = 4

    def hero_moving(self, screen, press_keys, block_group, window_x, window_y, WIN_BLOCK_FROM_END, GAME_TILE_SIZE):
        """
        use the block_group list created by GameMap.py
        :param screen: game screen
        :param press_keys: keys pressed on keyboard
        :param block_group: list of rectangles where wall squares are
        :return: is_over: True if hero hits the win block
        """
        #  assume the game is not over until proved otherwise
        is_over = False

        # end game on hitting escape
        if press_keys[K_ESCAPE]:
            exit()

        # if left arrow key pressed move to left by speed factor
        if press_keys[K_LEFT]:
            self.hero_rect.left -= self.hero_speed
            # has the character hit the edge of the screen, if so dont let past
            if self.hero_rect.left <= 0:
                self.hero_rect.left = 0
            # has the player hit a wall (in block group)
            for n in range(len(block_group)):
                # hero and block conflict
                if self.hero_rect.top > block_group[n].rect.top - GAME_TILE_SIZE and self.hero_rect.bottom < \
                        block_group[n].rect.bottom + GAME_TILE_SIZE:
                    if self.hero_rect.right > block_group[n].rect.right > self.hero_rect.left:
                        # If the character has hit a block - is it the win block?
                        if n == len(block_group) - WIN_BLOCK_FROM_END:
                            # set variable to return function game over
                            is_over = True
                        else:
                            # hit a normal block so undo the move
                            self.hero_rect.left += self.hero_speed

        if press_keys[K_RIGHT]:
            self.hero_rect.left += self.hero_speed
            if self.hero_rect.right >= window_x:
                self.hero_rect.right = window_x
            for n in range(len(block_group)):
                if self.hero_rect.top > block_group[n].rect.top - GAME_TILE_SIZE and self.hero_rect.bottom < \
                        block_group[n].rect.bottom + GAME_TILE_SIZE:
                    if self.hero_rect.left < block_group[n].rect.left < self.hero_rect.right:
                        if n == len(block_group) - WIN_BLOCK_FROM_END:
                            is_over = True
                        else:
                            self.hero_rect.left -= self.hero_speed

        if press_keys[K_UP]:
            self.hero_rect.top -= self.hero_speed
            if self.hero_rect.top <= 0:
                self.hero_rect.top = 0
            for n in range(len(block_group)):
                if self.hero_rect.left > block_group[n].rect.left - GAME_TILE_SIZE and self.hero_rect.right < \
                        block_group[n].rect.right + GAME_TILE_SIZE:
                    if self.hero_rect.bottom > block_group[n].rect.bottom > self.hero_rect.top:
                        if n == len(block_group) - WIN_BLOCK_FROM_END:
                            is_over = True
                        else:
                            self.hero_rect.top += self.hero_speed

        if press_keys[K_DOWN]:
            self.hero_rect.top += self.hero_speed
            if self.hero_rect.bottom >= window_y:
                self.hero_rect.bottom = window_y
            for n in range(len(block_group)):
                if self.hero_rect.left > block_group[n].rect.left - GAME_TILE_SIZE and self.hero_rect.right < \
                        block_group[n].rect.right + GAME_TILE_SIZE:
                    if self.hero_rect.top < block_group[n].rect.top < self.hero_rect.bottom:
                        if n == len(block_group) - WIN_BLOCK_FROM_END:
                            is_over = True  # winner
                        else:
                            self.hero_rect.top -= self.hero_speed

        screen.blit(self.hero_img[0], self.hero_rect)
        return is_over


    def monster_move(self, screen, hero_rect, block_group, move_rat, GAME_TILE_SIZE):
        """
        Move the monster in the direction of the character
        use the hero_rect for hero. self.hero_rect refers to the monster
        """
        isCaught = False


        # decide which direction to move in
        move_left = False
        move_right = False
        move_up = False
        move_down = False
        x_offset = self.hero_rect.left - hero_rect.left
        y_offset = self.hero_rect.top - hero_rect.top

        # move either left or right
        if x_offset > 0:
            move_left = True
        if x_offset < 0:
            move_right = True

        # move either up or down
        if y_offset > 0:
            move_up = True
        if y_offset < 0:
            move_down = True

        # MOVING LEFT
        if move_rat and move_left:
            #  was sel.hero_speed
            self.hero_rect.left -= 1

            # conflict with block
            for n in range(len(block_group)):  # block_group list is all block
                # hero and block conflict
                if self.hero_rect.top > block_group[n].rect.top - GAME_TILE_SIZE and self.hero_rect.bottom < \
                        block_group[n].rect.bottom + GAME_TILE_SIZE:
                    #
                    if self.hero_rect.right > block_group[n].rect.right > self.hero_rect.left:
                        self.hero_rect.left += 1  # if so, undo the move
        # MOVING RIGHT
        if move_rat and move_right:
            self.hero_rect.left += 1

            # conflict with block
            for n in range(len(block_group)):
                if self.hero_rect.top > block_group[n].rect.top - GAME_TILE_SIZE and self.hero_rect.bottom < \
                        block_group[n].rect.bottom + GAME_TILE_SIZE:
                    if self.hero_rect.left < block_group[n].rect.left < self.hero_rect.right:
                        self.hero_rect.left -= 1  # if so, undo the move
        # MOVIN UP
        if move_rat and move_up:
            self.hero_rect.top -= 1

            # conflict with block
            for n in range(len(block_group)):
                if self.hero_rect.left > block_group[n].rect.left - GAME_TILE_SIZE and self.hero_rect.right < \
                        block_group[n].rect.right + GAME_TILE_SIZE:
                    if self.hero_rect.bottom > block_group[n].rect.bottom > self.hero_rect.top:
                        self.hero_rect.top += 1  # if so, undo the move
        #
        # Movin DOWN
        if move_rat and move_down:
            self.hero_rect.top += 1

            # conflict with block
            for n in range(len(block_group)):
                if self.hero_rect.left > block_group[n].rect.left - GAME_TILE_SIZE and self.hero_rect.right < \
                        block_group[n].rect.right + GAME_TILE_SIZE:
                    if self.hero_rect.top < block_group[n].rect.top < self.hero_rect.bottom:
                        self.hero_rect.top -= 1 # if so, undo the move

        # transfer new position of hero
        screen.blit(self.hero_img[0], self.hero_rect)

        # if the monster has touched the character, then return function as true
        return isCaught




import tkinter as tk

"""
global variables (module scope)
nickname : store the nickname player input and print in the leaderboard
"""
nickname = ''
# Constants
STARTSCREEN_SIZE = "400x300"

"""
The startbox is the function before the main game, the player should input his nickname and click start button to 
start game
The close_the_window function is to connect to the start game button, close the start box and begin the game.
"""
def startbox():
    # inputbox width and height
    startscreen = tk.Tk()

    # title of the inputbox
    startscreen.title("")

    # startscreen.image = background_image     #
    def close_the_window():
        global nickname
        nickname = input_box_text.get()
        #close the start box
        startscreen.destroy()

    # top margin
    input_box_show_var = tk.Label(startscreen, height = 3, width = 50)
    input_box_show_var.pack(side=tk.TOP, expand=tk.NO)
    input_box_show_var["bg"] = "white"

    # show text
    show_var = "Welcome to Jeff's Lair"
    input_box_show_var = tk.Label(startscreen, text=show_var, width = 23, font=(None, 24))
    input_box_show_var.pack(side=tk.TOP, expand=tk.YES)
    input_box_show_var["bg"] = "white"

    # show text
    show_var = "Enter your name: "
    input_box_show_var = tk.Label(startscreen, text=show_var, width = 23, font=(None, 18))
    input_box_show_var.pack(side=tk.TOP, expand=tk.YES)
    input_box_show_var["bg"] = "white"

    # the entry text box
    input_box_text = tk.Entry(startscreen,  font=(None, 18))
    input_box_text.pack(anchor=tk.CENTER, expand=tk.YES)
    input_box_text["bg"] = None

    # padding
    input_box_show_var = tk.Label(startscreen, height = 1, width = 50)
    input_box_show_var.pack(side=tk.TOP, expand=tk.YES)
    input_box_show_var["bg"] = "white"

    # the button to start game
    start_button = tk.Button(startscreen, text="Start", width=20, font=(None, 18), command=lambda: close_the_window())
    start_button.pack(side=tk.TOP, expand=tk.YES)

    # bottom margin
    input_box_show_var = tk.Label(startscreen, height = 3, width = 50)
    input_box_show_var.pack(side=tk.TOP, expand=tk.YES)
    input_box_show_var["bg"] = "white"

    # show the box on the screen continuously
    tk.mainloop()
    return nickname



from distutils.core import setup

setup(
    name='JeffisLiar',
    packages = ['JeffisLiar'],
    version='1.0',
    license='MIT',
    description='JeffisLiar. Have fun with our game and help Jeff get out of the dungeon!',
    author='Binzhou Gu',
    author_email='bg548@bath.ac.uk',
    url='https://github.com/SE-Lab-Tim-Ray/dungeon-game/tree/installation',
    download_url='https://github.com/SE-Lab-Tim-Ray/dungeon-game/archive/V1.0.tar.gz',
    keywords=['DungeonGame', 'Joyful'],
    install_requires=[
        'pygame',
        'tkinter',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Games/Entertainment',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ],
)


import pygame

"""
GameMap should include(Version 1)
1.making block group for Hero and monsters to move
2.Map drawing
"""


# block class used to create array of rectangles for walls
class Block:
    def __init__(self, rect):
        self.rect = rect


class GameMap():
    def __init__(self, game_map_img, map_x, map_y):
        """
        :param game_map_img: map img
        :param map_x: map position x
        :param map_y: map position y
        """
        self.game_map_img = game_map_img
        self.map_x = map_x
        self.map_y = map_y

    def draw_map(self, screen):
        screen.blit(self.game_map_img, (self.map_x, self.map_y))

    # global block_group list is for Hero and monsters movement used in their own .py files
    def make_block_group(self, maze_map_file, TILE_SIZE):
        """
        label the block position in block_group list
        """
        block_group = []
        block_group.clear()

        # read the 1_maze.txt into lines as list
        with open(maze_map_file, 'r') as f:
            maze_lines = f.read().splitlines()

        # getting length of list
        num_lines = len(maze_lines)

        # assume all line lengths are the same as top line(columns)
        maze_map_width = len(maze_lines[0])
        maze_map_height = num_lines

        # based on the list, label the blocks in block_group list
        for i in range(maze_map_height):
            for j in range(maze_map_width):
                if maze_lines[i][j] == str(1):
                    block = Block(pygame.Rect(TILE_SIZE * j, TILE_SIZE * i, TILE_SIZE, TILE_SIZE))
                    block_group.append(block)

        return block_group

